from django.core.mail import send_mail

def send_notification_email(mail_from,mail_to, subject, message):
    send_mail(
        subject,
        message,
        mail_from,
        [mail_to],
        fail_silently=False,
    )





class Exception401(Exception):
    def __init__(self):
        self.message = "Unauthorized user or invalidate token"
        
    def __str__(self):
        return self.message
    

class Exception403(Exception):
    def __init__(self):
        self.message = "Unauthorized access to this resource"
        
    def __str__(self):
        return self.message


from azure.storage.blob import BlobServiceClient, ContentSettings, __version__
from azure.core import exceptions
from os import path, remove
from django_utils.local_file import write_file
from typing import BinaryIO
from django.conf import settings
from django_utils.logger_app import get_logger
import tempfile

BUCKET_NAME = settings.BUCKET_NAME

class BlobStorage:

    def __init__(self) -> None:
        self.logger = get_logger('AzureBlobStorage')
        try:
            self.logger.info(f"Azure Blob Storage v{__version__}")
            connection_string = settings.BLOB_STORAGE_CONNECTION_STRING
            self.blob_service_client = BlobServiceClient.from_connection_string(connection_string)
            self.container_client = self.blob_service_client.create_container(name=BUCKET_NAME, public_access='blob')

        except exceptions.ResourceExistsError as ex:
            self.logger.info(f"Container '{BUCKET_NAME}' already exists")

        except Exception as ex:
            self.logger.error(ex)

    def save_file(self, file, content_type) -> str:
        file_name = write_file(file, content_type)
        file_path = path.join(tempfile.gettempdir(), file_name)
        temp_file = open(file_path, mode='rb')
        self.upload_blob(temp_file, file_name, content_type)
        temp_file.close()
        remove(file_path)
        return file_name

    def upload_blob(self, source_file_name: BinaryIO, destination_blob_name: str, blob_type: str):
        blob_client = self.blob_service_client.get_blob_client(container=BUCKET_NAME, blob=destination_blob_name)

        # Upload the created file
        content_settings = ContentSettings(content_type=blob_type)
        blob_client.upload_blob(source_file_name, content_settings=content_settings)

        self.logger.info(f"File {source_file_name} uploaded to container {BUCKET_NAME}: {destination_blob_name}")

    def delete_blob(self, blob_name: str):
        blob_name = blob_name.replace(self.get_url_prefix(), "")

        blob_client = self.blob_service_client.get_blob_client(container=BUCKET_NAME, blob=blob_name)

        blob_client.delete_blob()

        self.logger.info(f"Blob {blob_name} deleted.")

    def get_url_prefix(self) -> str:
        return f"https://{settings.AZURE_STORAGE_ACCOUNT}.blob.core.windows.net/{BUCKET_NAME}/"


blob_storage = BlobStorage()


from functools import wraps
from django_client_sso.client_base import ClientBase
from django_client_sso.exceptions import Exception403,Exception401

def validator_with_request(permissions):
    def decorator_func(func):
        @wraps(func)
        def wrapper(self, request):
            client = ClientBase()
            token = request.META.get('HTTP_AUTHORIZATION')
            
            if not token:
                raise Exception401()
            
            token = token.replace('Bearer ', '')
            token_decode = client.decode_token(token)
            
            required = str(permissions).split()
            scope = token_decode['scope']

            for r in required:
                if r in scope:
                    return func(self,request)
           
            raise Exception403()
    
        return wrapper

    return decorator_func


def validator_with_args_kwargs(permissions):
    def decorator_func(func):
        @wraps(func)
        def wrapper(self, request, *args, **kwargs):
            client = ClientBase()
            token = request.META.get('HTTP_AUTHORIZATION')
            
            if not token:
                raise Exception401()
            
            token = token.replace('Bearer ', '')
            token_decode = client.decode_token(token)
            
            required = str(permissions).split()
            scope = token_decode['scope']

            for r in required:
                if r in scope:
                    return func(self,request, *args, **kwargs)
           
            raise Exception403()
    
        return wrapper

    return decorator_func

def validator_without_request(permissions):
    def decorator_func(func):
        @wraps(func)
        def wrapper(self):
            client = ClientBase()
            token = self.request.META.get('HTTP_AUTHORIZATION')
            
            if not token:
                raise Exception401()
            
            token = token.replace('Bearer ', '')
            token_decode = client.decode_token(token)
            
            required = str(permissions).split()
            scope = token_decode['scope']

            for r in required:
                if r in scope:
                    return func(self)
           
            raise Exception403()
    
        return wrapper

    return decorator_func



from rest_framework.views import exception_handler
from rest_framework.response import Response
from rest_framework import status
from django_client_sso.exceptions import Exception401,Exception403

def default_exception_handler(exc, context):
    response = exception_handler(exc, context)
    if response:
        return response
    
    if isinstance(exc, Exception401):
        return Response({'mensagem':str(exc.message)}, status=status.HTTP_401_UNAUTHORIZED)
        
    if isinstance(exc, Exception403):
        return Response({'mensagem':str(exc.message)}, status=status.HTTP_403_FORBIDDEN)
    
    return Response({'mensagem':str(exc.args)}, status=status.HTTP_400_BAD_REQUEST)
    
    


from azure.data.tables import TableServiceClient
from azure.core.exceptions import ResourceExistsError
from azure.core.credentials import AzureNamedKeyCredential
from azure.data.tables import TableTransactionError
from azure.core.exceptions import ResourceNotFoundError
from django.conf import settings
from django_utils.logger_app import get_logger


class AzureTableStorage:
    def __init__(self):
        self.logger = get_logger('AzureTableStorage')
        endpoint = f'https://{settings.AZURE_STORAGE_ACCOUNT}.table.core.windows.net'

        if settings.USE_AZURITE:
            endpoint = settings.AZURITE_CONNECTION_STRING

        credential = AzureNamedKeyCredential(name=settings.AZURE_STORAGE_ACCOUNT,key=settings.AZURE_ACCOUNT_KEY)
        self.table_service = TableServiceClient(endpoint, credential=credential)
        
        try:
            self.table_service.create_table(settings.TABLE_STORAGE)
        except ResourceExistsError:
            self.logger.info('Table existing in the database.')
        
        self.table_client = self.table_service.get_table_client(settings.TABLE_STORAGE)


    def get_entity_by_row_key(self, partition_key,row_key):
        try:
            return self.table_client.get_entity(partition_key, row_key)
        except ResourceNotFoundError:
            return None

    def query_entities(self, **kwargs):
        try:
            entities = self.table_client.query_entities(**kwargs)
            return entities
        except TableTransactionError as error:
            self.logger.error(f'Error when trying to search entity: {error}')
            return None
    def insert_entity(self, entity):
        try:
            return self.table_client.create_entity(entity=entity)
        except TableTransactionError as error:
            self.logger.error(f'Error when trying to insert entity: {error}')            

    def insert_or_replace_entity(self, entity):
        try:
            self.table_client.upsert_entity(mode='replace', entity=entity)
        except TableTransactionError as error:
            self.logger.error(f'Error when trying to update entity: {error}')            

    def update_entity(self, entity):
        try:
            self.table_client.update_entity(mode='merge', entity=entity)
        except TableTransactionError as error:
            self.logger.error(f'Error when trying to update entity: {error}')            

    def delete_entity(self, partition_key, row_key):
        try:
            self.table_client.delete_entity(partition_key, row_key)
        except TableTransactionError as error:
            self.logger.error(f'Error when trying to delete entity: {error}')            
            
class FilterBuilder:
    def __init__(self, partition_key):
        self.partition_key = partition_key
        self.filters = []

    def add_filter(self, field, operator, value):
        if value is not None:
            if isinstance(value, str):
                value = f"'{value}'"
            self.filters.append(f"{field} {operator} {value}")
        return self

    def build(self):
        filter_str = " and ".join(self.filters)
        return f"PartitionKey eq '{self.partition_key}' and {filter_str}" if filter_str else f"PartitionKey eq '{self.partition_key}'"


import logging

def get_logger(class_name:str):
    """ Formas de uso : info, warn, warning, error e debug
    
    Args:
        class_name : Nome da classe que será efetuado o log

    Returns:
        Logger : instance
    """
    logger = logging.getLogger(class_name)
    logger.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    formatter = logging.Formatter(
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    ch.setFormatter(formatter)
    logger.addHandler(ch)
    
    return logger


from abc import ABC, abstractmethod
import os
import importlib
from django_utils.logger_app import get_logger

class BaseSeeder(ABC):
    
    def __init__(self):
        self.logger = get_logger('BaseSeeder')
    
    @abstractmethod
    def run(self):
        pass

    def create_or_update(self, model, unique_field, unique_value, **defaults):
        unique_field_kwargs = {unique_field: unique_value}
        unique_field_kwargs.update(defaults)
        obj, created = model.objects.get_or_create(**unique_field_kwargs)
        if not created:
            self.logger.info(f"{self.__class__.__name__}: {model.__name__} with {unique_field} {unique_value} existing in the database.")
        else:
            self.logger.info(f"{self.__class__.__name__}: {model.__name__} with {unique_field} {unique_value} successfully created.")
        return obj, created

    @classmethod
    def run_all_seeders(cls, module_name):
        for filename in os.listdir(os.path.dirname(__file__)):
            if filename == "base_seeder.py" or not filename.endswith(".py"):
                continue

            module = importlib.import_module(f"{module_name}.{filename[:-3]}")
            seeder_class = None

            for name, obj in module.__dict__.items():
                if isinstance(obj, type) and issubclass(obj, BaseSeeder) and obj is not cls:
                    seeder_class = obj
                    break

            if seeder_class:
                seeder = seeder_class()
                seeder.run()

    @classmethod
    def run_specific_seeders(cls, module_name, seeder_filenames):
        for filename in seeder_filenames:
            module = importlib.import_module(f"{module_name}.{filename}")
            seeder_class = None

            for name, obj in module.__dict__.items():
                if isinstance(obj, type) and issubclass(obj, BaseSeeder) and obj is not cls:
                    seeder_class = obj
                    break

            if seeder_class:
                seeder = seeder_class()
                seeder.run()


from django.conf import settings
from azure.eventhub import EventData
from azure.eventhub.aio import EventHubProducerClient
from azure.eventhub.aio import EventHubConsumerClient
from azure.eventhub.aio._eventprocessor.partition_context import PartitionContext
from azure.eventhub.extensions.checkpointstoreblobaio import (
    BlobCheckpointStore,
)
import json


EVENT_HUB_NAME = settings.EVENT_HUB_NAME
EVENT_HUB_CONNECTION_STR = settings.EVENT_HUB_CONNECTION_STR
BLOB_STORAGE_CONNECTION_STRING = settings.BLOB_STORAGE_CONNECTION_STRING
BLOB_CONTAINER_NAME = settings.BLOB_CONTAINER_NAME

async def producer(data
                   ,eventhub_name = EVENT_HUB_NAME
                   ,eventhub_conn_str=EVENT_HUB_CONNECTION_STR):
    """
    Função que retorna a soma de dois números.

    Args:
        data (any): O primeiro número.
        eventhub_name(str): O segundo número.
        eventhub_conn_str (str) : terceiro elemento

    Returns:
        None
    """

    producer = EventHubProducerClient.from_connection_string(
        conn_str=eventhub_conn_str,
        eventhub_name=eventhub_name
    )

    async with producer:
        event_data_batch = await producer.create_batch()

        event = EventData(json.dumps(data))
        event.body_as_json(encoding='UTF-8')
        event_data_batch.add(event)
       
        await producer.send_batch(event_data_batch)

'''
async def on_event(partition_context:PartitionContext
                   , event:EventData):
   
    print(
        'Received the event: "{}" from the partition with ID: "{}"'.format(
            event.body_as_json(encoding="UTF-8"), partition_context.partition_id
        )
    )

    await partition_context.update_checkpoint(event)
'''

async def consumer(eventhub_name = EVENT_HUB_NAME
              ,eventhub_conn_str=EVENT_HUB_CONNECTION_STR
              ,blob_conn_str=BLOB_STORAGE_CONNECTION_STRING
              ,blob_name=BLOB_CONTAINER_NAME
              ,consumer_group='$Default'
              ,on_event=None):
    
    checkpoint_store = BlobCheckpointStore.from_connection_string(
        blob_conn_str, blob_name
    )

    client = EventHubConsumerClient.from_connection_string(
        eventhub_conn_str,
        consumer_group=consumer_group,
        eventhub_name=eventhub_name,
        retry_total=5,
        checkpoint_store=checkpoint_store
    )

    async with client:
       await client.receive(on_event=on_event)
        



import uuid
from os import path
import tempfile 

def write_file(file, content_type) -> str:
    file_name = '{}.{}'.format(uuid.uuid4(), content_type.split('/')[1])
    temp_full_path = path.join(tempfile.gettempdir(), file_name)
    contents = file.read()
    with open(temp_full_path, 'wb+') as f:
        f.write(contents)
        f.close()
    return file_name